# Name:   Convert-Csv2Ldif.ps1
# Author: Darvas Dniel 
# Desc:   Conversion tool from csvde tool input to openLdap ldapadd input
#	  (Creates LDIF file from CSV file.)
# Param:    $CsvPath - The name of the input csv file 
#           $NoPosixAccount - If set, no posixAccount attributes will be generated.
#           $uidNumber - First uidNumber.
# Conversion tool from csvde tool input to openLdap ldapadd input

param (
	[string] $CsvPath, #name of the csv file
	[switch] $NoPosixAccount,
	[int] $uidNumber = 2000 #first uidNumber
)

# FUNCTIONS
# WriteLdifFormat: writes the given string in LDIF format: if
#   the line is too long, it will be split into multiple rows
#   and after the first row the other rows will be started
#   with a space.
function WriteLdifFormat {
  param ([string]$str)

  $ROWMAXLEN = 60;
  $first = $true;
  while ($str.Length -gt 0) {
    $out = $str.Substring(0,[Math]::Min($ROWMAXLEN,$str.Length))
    if ($str.Length -gt $ROWMAXLEN)
    { $str = $str.Substring($ROWMAXLEN, $str.Length-$ROWMAXLEN) }
    else { $str = [string]::Empty }    
    
    #write to output
    if ($first)
    { write "$out"; $first=$false; $ROWMAXLEN--;}
    else { write " $out" }
  }
}

# WriteEmptyLine: Writes an empty line.
function WriteEmptyLine { write "" }

# LOADING CSV
$csv = Import-Csv -Path $CsvPath

# WRITE HEADER
WriteLdifFormat "# Converted from $CsvPath"
WriteLdifFormat "# Date: $(Get-Date)"
WriteLdifFormat "#   by $([Environment]::UserName) on $([Environment]::MachineName)"
WriteLdifFormat "#   using Convert-Csv2Ldif.ps1"
WriteEmptyLine 

# VARIABLE INIT
$rowNum = 0
$nobody = "cn=nobody,dc=irf,dc=local" #Dummy user for empty groups

$csv | foreach {
# iteration count
    $rowNum++
# drop empty rows
    if ($_.dn.Length -gt 5) {
# fixes
		# == fixing domain name ==
		$_.dn = $_.dn.Replace("dc=irfhf,","dc=irf,")
		$_.member = $_.member.Replace("dc=irfhf,","dc=irf,")
		# rembember: the nobody user also contains the domain name! 
	
		# == common infos ==
		WriteLdifFormat "# from: $CsvPath row: $rowNum"
		WriteLdifFormat "dn: $($_.dn)"
		WriteLdifFormat "objectClass: top"

		# == specific infos ==
		if ($_.objectClass -eq "organizationalUnit") {
			WriteLdifFormat "objectClass: organizationalUnit"
			WriteLdifFormat "ou: $($_.ou)"
			WriteLdifFormat "description: $($_.name) Organizational Unit"
			
			#IRF specific part
			# create OU's staff group
			WriteEmptyLine
			WriteLdifFormat "dn: cn=$($_.ou)staff,$($_.dn)"
			WriteLdifFormat "objectClass: groupOfNames"
			WriteLdifFormat "cn: $($_.ou)staff"
			WriteLdifFormat "description: $($_.name) Staff"
			WriteLdifFormat "member: $nobody" #dummy user
		}
		if ($_.objectClass -eq "user") {
			WriteLdifFormat "cn: $($_.samaccountname)"
			WriteLdifFormat "objectClass: inetOrgPerson"
			WriteLdifFormat "objectClass: person"
			WriteLdifFormat "displayName: $($_.name)"
			WriteLdifFormat "sn: $($_.sn)"
			WriteLdifFormat "givenName: $($_.givenName)"
			WriteLdifFormat "mail: $($_.mail)"
			WriteLdifFormat "telephoneNumber: $($_.telephoneNumber)"
			WriteLdifFormat "description: $($_.description)"
			#posixAccount
			if ($NoPosixAccount -eq $false) {
				WriteLdifFormat "objectClass: posixAccount"
				WriteLdifFormat "uid: $($_.samaccountname)"
				WriteLdifFormat "uidNumber: $uidNumber"
				WriteLdifFormat "gidNumber: 1000" # not so brave...
				WriteLdifFormat "homeDirectory: /home/users/$($_.samaccountname)"
				WriteLdifFormat "loginShell: /bin/sh"
				$uidNumber++
			}
			# IRF specific part
			# add to the OU's staff group
			if ($_.department.Length -gt 0) {
				WriteEmptyLine
				$parentgroup = $_.DN.Replace($_.samaccountname,$_.department + "staff")
				WriteLdifFormat "dn: $parentgroup"
				WriteLdifFormat "changetype: modify"
				WriteLdifFormat "add: member"
				WriteLdifFormat "member: $($_.dn)"
			}
		}
		if ($_.objectClass -eq "group") {
			WriteLdifFormat "cn: $($_.cn)"
			WriteLdifFormat "objectClass: groupOfNames"
			$_.member.Split(';') | % { WriteLdifFormat "member: $_" }
		}

		WriteEmptyLine 
		WriteEmptyLine 
    } #endif
}

WriteEmptyLine 
WriteEmptyLine 